#!/usr/bin/perl
#
# Jonathan Klassen
#
# Aligned_multiple_faa_concatenator.pl
# v1.1 - Sept. 6, 2012
#
# Script for concatenating a series of aligned multiple fasta files into a single alignment

use warnings;
use Getopt::Long;

######################################################################################################################
# Processes input arguments
######################################################################################################################

my $usage = "Aligned_multiple_faa_concatenator.pl

DEPENDANCIES: None

USAGE:
-a	skip removal of mostly gapped columns (Y or N)			DEFAULT: N	e.g., perl Aligned_multiple_faa_concatenator.pl -a Y
-b	minimum decimal percentage of column containing gaps to include DEFAULT: 0.02	e.g., perl Aligned_multiple_faa_concatenator.pl -b 0.05
-c	substitution matrix name for RAxML partition file		DEFAULT: GTR	e.g., perl Aligned_multiple_faa_concatenator.pl -c WAG
-h	displays this usage statement (also using --help)
-i	input directory containing only the input aligned faa files 	DEFAULT: Mult_MUSCLE_corrected_multiple_faas		
		e.g., perl Aligned_multiple_faa_concatenator.pl -i otherdir
IMPORTANT: faa headers in these files must have the format \">(number, word or \"_\") (organism name)\"; otherwise this script WILL NOT WORK CORRECTLY
-q	run quietly, i.e., no STOUT (Y or N)				DEFAULT: N 	e.g., perl Aligned_multiple_faa_concatenator1.pl -q N

OUTPUT FILES:
	Aligned_multiple_faa_concatenator.faa			Concatenated multiple fasta file
	Aligned_multiple_faa_concatenator.partitions		Partitions file for RAxML input, each protein type a separate partition
	Aligned_multiple_faa_concatenator.faa.degapped		Concatenated multiple fasta file with columns conataining gaps above the given
									threshold removed
	Aligned_multiple_faa_concatenator.partitions.degapped	Partitions file for RAxML input with columns contatining gaps above the given
									threshold removed, each protein type a separate partition
";

# input arguments

%options = ();
GetOptions(

	"a=s" => \$options{skip_column_removal},		"b=s" => \$options{skip_fraction},
	"c=s" => \$options{substitution_matrix},		"h" => \$options{help},
	"help" => \$options{help}, 				"i=s" => \$options{input_faa_dir},
	"q=s" => \$options{quiet}
);

# display usage statement if called
 
die $usage if ($options{help});

# default arguments

unless ($options{skip_column_removal}){ $options{skip_column_removal} = "Y" }
unless ($options{substitution_matrix}){ $options{substitution_matrix} = "GTR" }
unless ($options{skip_fraction}){ $options{skip_fraction} = 0.02 }
unless ($options{input_faa_dir}){ $options{input_faa_dir} = "Mult_MUSCLE_corrected_multiple_faas" }
unless ($options{quiet}){ $options{quiet} = "N" }

# mystery input flags not allowed

die "Unrecognized command line arguments: @ARGV\n" if ($ARGV[0]);

# checks correct parameter formatting

die "Unrecognized command line arguments: -a = $options{quiet}\n$usage" 
	unless ($options{skip_column_removal} eq "Y" or $options{skip_column_removal} eq "N");
die "Unrecognized command line arguments: -b = $options{skip_fraction}\n$usage" unless ($options{skip_fraction} =~ /^0\.\d+$/);
die "Unrecognized command line arguments: -q = $options{quiet}\n$usage" unless ($options{quiet} eq "Y" or $options{quiet} eq "N");

# checks if input and output files exist

unless (-d $options{input_faa_dir}){ die "$options{input_faa_dir} does not exist\n$usage" }
unless ($options{quiet} eq "N"){
	if (-e "Aligned_multiple_faa_concatenator.faa"){ 
		print "WARNING: Aligned_multiple_faa_concatenator.faa file exists and has been overwritten\n" }
	if (-e "Aligned_multiple_faa_concatenator.partitions"){ 
		print "WARNING: Aligned_multiple_faa_concatenator.partitions file exists and has been overwritten\n" }
	if (-e "Aligned_multiple_faa_concatenator.faa.degapped"){ 
		print "WARNING: Aligned_multiple_faa_concatenator.faa.degapped file exists and has been overwritten\n" }
	if (-e "Aligned_multiple_faa_concatenator.partitions.degapped"){ 
		print "WARNING: Aligned_multiple_faa_concatenator.partitions.degapped file exists and has been overwritten\n" }
}
if (-e "Aligned_multiple_faa_concatenator.faa"){ system ("rm Aligned_multiple_faa_concatenator.faa") }
if (-e "Aligned_multiple_faa_concatenator.partitions"){ system ("rm Aligned_multiple_faa_concatenator.partitions") }
if (-e "Aligned_multiple_faa_concatenator.faa.degapped"){ system ("rm Aligned_multiple_faa_concatenator.faa.degapped") }
if (-e "Aligned_multiple_faa_concatenator.partitions.degapped"){ system ("rm Aligned_multiple_faa_concatenator.partitions.degapped") }

# print parameters unless -q flag selected

print "
----------------------------------------------------------------
Aligned_multiple_faa_concatenator.pl	Jonathan Klassen 	v1.2: Dec. 13, 2012

parameters used:
	exclude columns comprised mostly of gaps = $options{skip_column_removal}
	minimum percent of a column not gaps to include = $options{skip_fraction}
	substitution matrix name for RAxML partition file = $options{substitution_matrix}
	input directory containing only the input aligned faa files = $options{input_faa_dir}
	run quietly = $options{quiet}
----------------------------------------------------------------
" if ($options{quiet} eq "N");

######################################################################################################################
# Loads input aligned multiple fasta files
######################################################################################################################

# load file names and derive organism names

my (@files, @names, $temp);
opendir (DIR, $options{input_faa_dir}) or die "Cannot opendir $options{input_faa_dir}";
@files = readdir DIR;
foreach (@files){
	chomp;
	($temp = $_) =~ s/\..*$//;
	push @names, $temp;
}

# load faa sequences, determine their lengths

my (%all_seqs, %length_infile, $organism_name);
for my $a (0..$#files){
	next if ($files[$a] =~ /^\./);
	open (INFILE, "$options{input_faa_dir}/$files[$a]") or die "Cannot open $options{input_faa_dir}/$files[$a]\n$usage";
	while (<INFILE>){
		chomp;
		next unless ($_ =~ /\S/);
		if (/>/){
			/>\S+ (\S+)/ or die "Cannot match fasta header in the multiple fasta infile\n$usage";
			$organism_name = $1;
		}
		else { 
			if (!$all_seqs{$organism_name}{$names[$a]}){ $all_seqs{$organism_name}{$names[$a]} = $_ }
			else {$all_seqs{$organism_name}{$names[$a]} = $all_seqs{$organism_name}{$names[$a]}.$_ }
			$length_infile{$names[$a]} = length $all_seqs{$organism_name}{$names[$a]};
		}
	}
}

# If a gene is missing in an organism, add it comprised on X's over the entire aligned length

foreach $name (sort keys %all_seqs){
	foreach $infile (sort @names){
		if (!$all_seqs{$name}{$infile}){
			if (!$length_infile{$infile}){
				$length_infile{$infile} = 0;
			}
			$all_seqs{$name}{$infile} = "X" x $length_infile{$infile};
		}
	}
}

######################################################################################################################
# Creates partition output file for use by RAxML
######################################################################################################################

open (OUTFILE, ">Aligned_multiple_faa_concatenator.partitions") or die "Cannot open Aligned_multiple_faa_concatenator.partitions";
my $base_counter = 1;
foreach $name (sort keys %all_seqs){
	foreach $infile (sort keys %{$all_seqs{$name}}){
		next unless ($infile =~ /\w/);
		print OUTFILE "$options{substitution_matrix}, $infile = $base_counter-", 
			($base_counter += length $all_seqs{$name}{$infile}) - 1, "\n";
	}
	last;
}

######################################################################################################################
# Creates concatenated multiple protein fasta file
######################################################################################################################

open (OUTFILE, ">Aligned_multiple_faa_concatenator.faa") or die "Cannot open Aligned_multiple_faa_concatenator.faa";
foreach $name (sort keys %all_seqs){
	print OUTFILE ">$name\n";
	foreach $infile (sort keys %{$all_seqs{$name}}){ 
		next unless ($infile =~ /\w/);
		print OUTFILE $all_seqs{$name}{$infile}, "\n"; 
	}
}

######################################################################################################################
# Removes mostly-gap columns
######################################################################################################################

if ($options{skip_column_removal} eq "N"){
	if ($options{quiet} eq "N"){ print "\nRemoving mostly gapped positions\n" }

	open (INFILE, "Aligned_multiple_faa_concatenator.faa") or die "Cannot open Aligned_multiple_faa_concatenator.faa";
	open (OUTFILE, ">Aligned_multiple_faa_concatenator.faa.degapped") or die "Cannot open Aligned_multiple_faa_concatenator.faa.degapped";
	open (PARTITIONSIN, "Aligned_multiple_faa_concatenator.partitions") or die "Cannot open Aligned_multiple_faa_concatenator.partitions";
	open (PARTITIONSOUT, ">Aligned_multiple_faa_concatenator.partitions.degapped") or die "Aligned_multiple_faa_concatenator.partitions.degapped";

	# reads through concatenated .faa file, storing the headers in one array and the alignment in another

	my ($a, @line, @headers, @outarray, $b, $count, %array_positions_to_skip);
	$a = -1;
	while (<INFILE>){
		@line = ();
		if (/>/){ 
			push @headers, $_;
			$a++;
			next;
		}
		else {
			chomp;
			@line = split '';
			foreach (@line){
				push @{$outarray[$a]}, $_;
			}
		}
	}
	
	# determines which columns pass the cutoff threshold

	for $b (0..$#{$outarray[0]}){ 
		$count = 0;
		for $a (0..$#outarray){
		
			if ($outarray[$a][$b] eq "-" or $outarray[$a][$b] eq "X"){
				$count++;
			}
		}
		if (($#outarray + 1 - $count) / ($#outarray + 1) < $options{skip_fraction}){
			$array_positions_to_skip{$b} = "y";
		}
	}	

	# reads through the concatenated .faa file again, keeping the fasta headers and only outputting alignment columns having content > 
	# the cutoff threshold

	for $a (0..$#headers){
		print OUTFILE $headers[$a];
		for $b (0..$#{$outarray[$a]}){
			next if ($array_positions_to_skip{$b});
			print OUTFILE $outarray[$a][$b];
		}
		print OUTFILE "\n";
	}
	
	# reads through partitions.out file, correcting for deleted positions

	my ($correction, $model, $gene_name, $start, $end, $deleted, $c);
	$correction = 0;
	while (<PARTITIONSIN>){
		/(\w+), (\w+) = (\d+)-(\d+)/ or die "Cannot match partition";
		$model = $1;
		$gene_name = $2;
		$start = $3;
		$end = $4;
		$deleted = 0;
		for $c ($start - 1 .. $end - 1){
			if ($array_positions_to_skip{$c}){
				$deleted++;
			}
		}
		print PARTITIONSOUT "$model, $gene_name = ", $start - $correction, "-", $end - $correction - $deleted, "\n";
		$correction += $deleted;
	}
}
