#!/usr/bin/perl
#
# Jonathan Klassen
#
# Mult_MUSCLE.pl
# v1.2 - Dec 18, 2012
#
# Wrapper for making MUSCLE alignments
# Multithreaded, one thread for each multiple alignment considered

use strict;
use warnings;
use Parallel::ForkManager;
use Getopt::Long;

######################################################################################################################
# Processes input arguments
######################################################################################################################

my $usage = "Mult_MUSCLE.pl

DEPENDANCIES: BLAST+ suite, MUSCLE, Perl \"Parallel::ForkManager\" module

USAGE:
-c 	number of cores to use					DEFAULT: 1				  e.g., perl Mult_MUSCLE.pl -i inlist -c 4
-h	displays this usage statement (also using --help)
-i 	input list of paths to the fasta files to be analyzed	DEFAULT: Mult_corrected_faa.list	  e.g., perl Mult_MUSCLE.pl -i inlist
-q	run quietly, i.e., no STDOUT (Y or N)			DEFAULT: N				  e.g., perl Mult_MUSCLE.pl -i inlist -q N

OUTPUT FILES:
	Mult_MUSCLE_corrected_multiple_faas			Directory containing corrected aligned multiple fasta files
	Mult_MUSCLE_corrected_multiple_faas.list		List of corrected aligned multiple fasta files
";

# input arguments

my %options = ();
GetOptions(
	"c=i" => \$options{cores}, 		"h" => \$options{help},
	"help" => \$options{help}, 		"i=s" => \$options{input_faa_list},
	"q=s" => \$options{quiet}
);

# display usage statement if called
 
die $usage if ($options{help});

# default arguments

unless ($options{cores}){ $options{cores} = 1 }
unless ($options{input_faa_list}){ $options{input_faa_list} = "Mult_corrected_faa.list" }
unless ($options{quiet}){ $options{quiet} = "N" }

# mystery input flags not allowed

die "Unrecognized command line arguments: @ARGV\n" if ($ARGV[0]);

# checks correct parameter formatting

die "Unrecognized command line arguments: -q = $options{quiet}\n$usage" unless ($options{quiet} eq "Y" or $options{quiet} eq "N");

# checks if input and output files exist

unless (-e $options{input_faa_list}){ die "$options{input_faa_list} does not exist\n$usage" }
if ($options{quiet} eq "N"){
	if (-e "Mult_MUSCLE_corrected_multiple_faas.list"){ 
		print "WARNING: Mult_MUSCLE_corrected_multiple_faas.list file exists and has been overwritten\n";
	}
	if (-d "Mult_MUSCLE_corrected_multiple_faas"){
		print "WARNING: Mult_MUSCLE_corrected_multiple_faas output directory exists and has been overwritten\n";
	}
}
if (-e "Mult_MUSCLE_corrected_multiple_faas.list"){ system "rm Mult_MUSCLE_corrected_multiple_faas.list" }
if (-d "Mult_MUSCLE_corrected_multiple_faas"){ system "rm -rf Mult_MUSCLE_corrected_multiple_faas" }

# print parameters unless -q flag selected

print "
----------------------------------------------------------------
Mult_MUSCLE.pl		Jonathan Klassen 	v1.2: Dec 18, 2012

parameters used:
	number of cores to use = $options{cores}
	input list of multiple faa files, one for each protein type = $options{input_faa_list}
	run quietly = $options{quiet}
----------------------------------------------------------------
" if ($options{quiet} eq "N");

# makes a new one output directory

mkdir "Mult_MUSCLE_corrected_multiple_faas";

######################################################################################################################
# Loads input files, derives names from them
######################################################################################################################

open (INFILELIST, $options{input_faa_list}) or die "Cannot open $options{input_faa_list}";

our (@input_faa_file_list, @input_faa_file_names);
while (<INFILELIST>){
	chomp;			
	push @input_faa_file_list, $_;
	/\/((\w|-)+)\..*$/;
	my $temp = $1;
	$temp =~ s/.*\///;
	$temp =~ s/mult_//;
	push @input_faa_file_names, $temp;
}

######################################################################################################################
# Executes MUSCLE subroutine
######################################################################################################################

my @outfile_names;

sub muscle_executer($);

our %summary_output;

my $pm = new Parallel::ForkManager($options{cores});

for my $a (0..$#input_faa_file_list){
	$pm->start and next;
	my @return = muscle_executer($a);

	unless (-d "temp_outputs"){ mkdir "temp_outputs" }
	open (TEMPOUT, ">temp_outputs/$a.tempout") or die "Cannot open temp_outputs/$a.tempout";
	print TEMPOUT join "\t", @return;
	close TEMPOUT;
	$pm->finish;
	push @outfile_names, "Mult_MUSCLE_corrected_multiple_faas/$input_faa_file_names[$a]_corrected_aligned.faa";
}
$pm->wait_all_children();

######################################################################################################################
# Creates output file list
######################################################################################################################

open (OUTFILE, ">Mult_MUSCLE_corrected_multiple_faas.list") or die "Cannot open Mult_MUSCLE_corrected_multiple_faas.list";
foreach (@outfile_names){
	print OUTFILE $_, "\n";
}

######################################################################################################################
# Subroutine to execute MUSCLE
######################################################################################################################

sub muscle_executer ($){
	my $counter = $_[0]; # index counter for global vars
	if ($options{quiet} eq "N"){ print "Aligning: $input_faa_file_names[$counter]\n" }
	system "muscle -in $input_faa_file_list[$counter] -out Mult_MUSCLE_corrected_multiple_faas/$input_faa_file_names[$counter]_corrected_aligned.faa -quiet";
}

